﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SharpGL;

namespace OpenGLHouse
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            InitializeOpenGl();
        }

        private void OnExit(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void InitializeOpenGl()
        {
            OpenGlControl.FrameRate = 25.0f;
            var gl = OpenGlControl.OpenGL;

            gl.ClearColor(0.0f, 0.0f, 0.0f, 0.0f);
            gl.Enable(OpenGL.DEPTH_TEST);
            gl.Viewport(0, 0, OpenGlControl.Width, OpenGlControl.Height);
            gl.MatrixMode(OpenGL.PROJECTION);
            gl.LoadIdentity();

            if (Perspective)
            {//Perspectivic Rendering
                OpenGlControl.OpenGL.Perspective(
                45.0d, (double)OpenGlControl.Width / (double)OpenGlControl.Height,
                0.1d, 100.0d);
            }
            else
            {// Orthogonal Rendering
                OpenGlControl.OpenGL.Ortho(-3, 3, -3, 3, 2, 10);
            }

            gl.LookAt(
                0.0d, 2.0d, 8.0d,   // eye
                0.0d, 2.0d, -1.0d,  // center
                0.0d, 1.0d, 0.0d);  // up

            gl.MatrixMode(OpenGL.MODELVIEW);
        }

        private float _HorizontalRotationAngle = 0.0f;
        private float HorizontalRotationAngle
        {
            get
            {
                return _HorizontalRotationAngle;
            }
            set
            {
                _HorizontalRotationAngle = value;
            }
        }

        private float _VerticalRotationAngle = 0.0f;
        private float VerticalRotationAngle
        {
            get
            {
                return _VerticalRotationAngle;
            }
            set
            {
                _VerticalRotationAngle = value;
            }
        }

        private float _DeepRotationAngle = 0.0f;
        private float DeepRotationAngle
        {
            get
            {
                return _DeepRotationAngle;
            }
            set
            {
                _DeepRotationAngle = value;
            }
        }

        private float _ZPosition = 0.0f;
        private float ZPosition
        {
            get
            {
                return _ZPosition;
            }
            set
            {
                _ZPosition = value;
            }
        }

        private float _YPosition = 0.0f;
        private float YPosition
        {
            get
            {
                return _YPosition;
            }
            set
            {
                _YPosition = value;
            }
        }

        private float _XPosition = 0.0f;
        private float XPosition
        {
            get
            {
                return _XPosition;
            }
            set
            {
                _XPosition = value;
            }
        }

        private bool _FirstTranslateThanRotate = true;
        private bool FirstTranslateThanRotate
        {
            get
            {
                return _FirstTranslateThanRotate;
            }
            set
            {
                _FirstTranslateThanRotate = value;
            }
        }

        private bool _Perspective = true;
        private bool Perspective
        {
            get
            {
                return _Perspective;
            }
            set
            {
                _Perspective = value;
            }
        }

        private void OpenGlDraw(object sender, PaintEventArgs args)
        {
            var gl = OpenGlControl.OpenGL;
            gl.Clear(OpenGL.COLOR_BUFFER_BIT | OpenGL.DEPTH_BUFFER_BIT);
            gl.LoadIdentity();

            if(FirstTranslateThanRotate)
            {
                gl.Translate(XPosition, YPosition, ZPosition);
                gl.Rotate(VerticalRotationAngle, HorizontalRotationAngle, DeepRotationAngle);
            }
            else
            {
                gl.Rotate(VerticalRotationAngle, HorizontalRotationAngle, DeepRotationAngle);
                gl.Translate(XPosition, YPosition, ZPosition);
            }

            gl.Begin(OpenGL.TRIANGLES);

            #region Front
            gl.Color(0.0f, 1.0f, 0.0f);
            // front, upper/left
            gl.Vertex(-1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(-1.0f, 0.0f, 1.0f);
            // front, lower/right
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 0.0f, 1.0f);
            gl.Vertex(-1.0f, 0.0f, 1.0f);
            //hat
            gl.Color(1.0f, 1.0f, 0.0f);
            gl.Vertex(-1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(0.0f, 4.0f, 0.0f);
            #endregion

            #region Back
            gl.Color(0.0f, 0.5f, 0.0f);
            // back, upper/left
            gl.Vertex(-1.0f, 2.0f, -1.0f);
            gl.Vertex(1.0f, 2.0f, -1.0f);
            gl.Vertex(-1.0f, 0.0f, -1.0f);
            // back, lower/right
            gl.Vertex(1.0f, 2.0f, -1.0f);
            gl.Vertex(1.0f, 0.0f, -1.0f);
            gl.Vertex(-1.0f, 0.0f, -1.0f);
            //hat
            gl.Color(0.5f, 0.5f, 0.0f);
            gl.Vertex(1.0f, 2.0f, -1.0f);
            gl.Vertex(-1.0f, 2.0f, -1.0f);
            gl.Vertex(0.0f, 4.0f, 0.0f);
            #endregion

            #region Left
            gl.Color(0.0f, 0.75f, 0.0f);
            // left; upper/back
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 2.0f, -1.0f);
            gl.Vertex(1.0f, 0.0f, -1.0f);
            //left; lower/front
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 0.0f, -1.0f);
            gl.Vertex(1.0f, 0.0f, 1.0f);
            //hat
            gl.Color(0.75f, 0.75f, 0.0f);
            gl.Vertex(-1.0f, 2.0f, 1.0f);
            gl.Vertex(-1.0f, 2.0f, -1.0f);
            gl.Vertex(0.0f, 4.0f, 0.0f);
            #endregion

            #region Right
            gl.Color(0.0f, 0.75f, 0.0f);
            // left; upper/back
            gl.Vertex(-1.0f, 2.0f, 1.0f);
            gl.Vertex(-1.0f, 2.0f, -1.0f);
            gl.Vertex(-1.0f, 0.0f, -1.0f);
            //left; lower/front
            gl.Vertex(-1.0f, 2.0f, 1.0f);
            gl.Vertex(-1.0f, 0.0f, -1.0f);
            gl.Vertex(-1.0f, 0.0f, 1.0f);
            //hat
            gl.Color(0.75f, 0.75f, 0.0f);
            gl.Vertex(1.0f, 2.0f, 1.0f);
            gl.Vertex(1.0f, 2.0f, -1.0f);
            gl.Vertex(0.0f, 4.0f, 0.0f);
            #endregion

            #region Bottom
            gl.Color(0.0f, 0.25f, 0.0f);
            // bottom, left/front
            gl.Vertex(-1,0,1);
            gl.Vertex(1,0,1);
            gl.Vertex(-1,0,-1);

            // bottom, right/back
            gl.Vertex(1,0,-1);
            gl.Vertex(1, 0, 1);
            gl.Vertex(-1, 0, -1);
            #endregion

            gl.End();
            gl.SwapBuffers();
        }

        #region Projection
        private void OnProjectionToPerspective(object sender, EventArgs e)
        {
            PerspectiveMenuItem.Checked = true;
            OrthogonalMenuItem.Checked = false;
            Perspective = true;
            InitializeOpenGl();
        }

        private void OnProjectionToOrthogonal(object sender, EventArgs e)
        {
            PerspectiveMenuItem.Checked = false;
            OrthogonalMenuItem.Checked = true;
            Perspective = false;
            InitializeOpenGl();
        }
        #endregion

        #region RotationTranslationMode
        private void OnRotationThanTranslation(object sender, EventArgs e)
        {
            FirstTranslateThanRotate = true;
            FirstRotateThanTranslateMenuItem.Checked = true;
            FirstTranslateThanRotateMenuItem.Checked = false;
        }

        private void OnTranslationThanRotation(object sender, EventArgs e)
        {
            FirstTranslateThanRotate = false;
            FirstRotateThanTranslateMenuItem.Checked = false;
            FirstTranslateThanRotateMenuItem.Checked = true;
        }
        #endregion

        #region Rotate
        private void OnRotateLeft(object sender, EventArgs e)
        {
            HorizontalRotationAngle += 5.0f;
        }

        private void OnRotateRight(object sender, EventArgs e)
        {
            HorizontalRotationAngle -= 5.0f;
        }

        private void OnRotateUp(object sender, EventArgs e)
        {
            VerticalRotationAngle += 5.0f;
        }

        private void OnRotateDown(object sender, EventArgs e)
        {
            VerticalRotationAngle -= 5.0f;
        }

        private void OnRotateCounterclockwise(object sender, EventArgs e)
        {
            DeepRotationAngle += 5.0f;
        }
        private void OnRotateClockwise(object sender, EventArgs e)
        {
            DeepRotationAngle -= 5.0f;
        }
        #endregion

        #region Translate
        private void OnTranslateRight(object sender, EventArgs e)
        {
            XPosition += 0.1f;
        }

        private void OnTranslateLeft(object sender, EventArgs e)
        {
            XPosition -= 0.1f;
        }

        private void OnTranslateUp(object sender, EventArgs e)
        {
            YPosition += 0.1f;
        }

        private void OnTranslateDown(object sender, EventArgs e)
        {
            YPosition -= 0.1f;
        }

        private void OnTranslateForward(object sender, EventArgs e)
        {
            ZPosition += 0.1f;
        }
        private void OnTranslateBack(object sender, EventArgs e)
        {
            ZPosition -= 0.1f;
        }
        #endregion

        private void OnResetView(object sender, EventArgs e)
        {
            YPosition = 0.0f;
            XPosition = 0.0f;
            ZPosition = 0.0f;
            HorizontalRotationAngle = 0.0f;
            VerticalRotationAngle = 0.0f;
            DeepRotationAngle = 0.0f;
        }

        private void OnMainWindowSizeChanged(object sender, EventArgs e)
        {
            InitializeOpenGl();
        }
    }
}
